//==============================================================================
/**
 * @file    multi_tree.hpp
 * @brief   多分木
 * @author  H.I.
 * @date    2011-02-14
 * @version $Id: multi_tree.hpp 40 2011-06-19 16:51:40Z blue $
 */
//==============================================================================


#ifndef HIL_MULTI_TREE_HPP
#define HIL_MULTI_TREE_HPP


//==============================================================================
// include
//==============================================================================
#include <iostream>
#include <string>

HIL_BEGIN_NAMESPACE;

namespace common {
//==============================================================================
// define
//==============================================================================

//==============================================================================
/**
 * 多分木ノード
 *
 * @tparam 数値パラメータ
 */
//==============================================================================
template <typename tType>
class clMultiTree {
public:
  typedef tType value_type;
  typedef clMultiTree<value_type> self_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clMultiTree()
  : m_delete(true),
    m_parent(0),
    m_next(0),
    m_child_top(0),
    m_child_end(0),
    m_value()
  {
  }

  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clMultiTree()
  {
    popAll();
  }
  
  //-----------------------------------------------------------------------
  /**
   * ノードの検索
   *
   * @param[ in ] hvalue  : 検索数値
   * return 見つかったノード
   */
  //-----------------------------------------------------------------------
  self_type* find(const value_type& hvalue) {
    if (m_value == hvalue)  return this;
    
    for (self_type* node(getChildTop()) ; node != 0 ; node = node->getSibling()) {
      self_type* ret = node->find(hvalue);
      if (ret)  return ret;
    }
    return 0;
  }
  
  //-----------------------------------------------------------------------
  /**
   * ノードの検索
   *
   * @param[ in ] hvalue  : 検索数値
   * return 見つかったノード
   */
  //-----------------------------------------------------------------------
  const self_type* find(const value_type& hvalue) const {
    if (m_value == hvalue)  return this;
    
    for (self_type* node(getChildTop()) ; node != 0 ; node = node->getSibling()) {
      self_type* ret = node->find(hvalue);
      if (ret)  return ret;
    }
    return 0;
  }
  
  
  
  /// ノード差し替え時にメモリ破棄するか
  void setDelete(bool hflag)  { m_delete = hflag; }
  /// 末尾にノードを追加
  void pushBack(self_type* hnode)
  {
    if (m_child_top == 0) {
      m_child_top = hnode;
    }
    if (m_child_end) {
      m_child_end->setNext(hnode);
    }
    m_child_end = hnode;
    hnode->setParent(this);
  }
  /// 数値を設定
  void setValue(const value_type& hvalue) { m_value = hvalue; }
  
  
  
  /// 全てのノードを削除
  void popAll()
  {
    self_type* node(m_child_top);
    while (node) {
      popFront();
      node = m_child_top;
    }
  }
  /// 指定ノードを削除する
  void popChild(self_type* hnode)
  {
    self_type* node(m_child_top);
    self_type* last(0);
    while (node) {
      if (node == hnode) {
        if (last) {
          last->setNext(node->getSibling());
        }
        if (m_child_top == node) {
          m_child_top = node->getSibling();
        }
        if (m_child_end == node) {
          m_child_end = last;
        }
//        last->setNext(node->getSibling());
        node->setParent(0);
        if (m_delete) delete node;
        break;
      }
      last = node;
      node = node->getSibling();
    }
  }
  /// 先頭のノードを削除
  void popFront()
  {
    popChild(m_child_top);
  }
  /// 終端のノードを削除
  void popBack()
  {
    popChild(m_child_end);
  }
  /// 親ノードの取得
  self_type* getParent() const { return m_parent; }
  /// 兄弟ノードの取得
  self_type* getSibling() const { return m_next; }
  /// 先頭ノードの取得
  self_type* getChildTop() const { return m_child_top; }
  /// 末尾ノードの取得
  self_type* getChildEnd() const { return m_child_end; }
  /// 数値を取得
  const value_type& getValue() const { return m_value; }
  /// 次のノードを取得
  self_type* getNext(bool hsearch_child=true) const
  {
    if (getChildTop() && hsearch_child) {
      return getChildTop();
    }
    if (getSibling()) {
      return getSibling();
    }
    if (getParent()) {
      return getParent()->getNext(false);
    }
    return 0;
  }


  
protected:
  /// 親ノードを設定する
  void setParent(self_type* hnode)  { m_parent = hnode; }
  /// 兄弟ノードを設定する
  void setNext(self_type* hnode)  { m_next = hnode; }
  
// private:
public:
  bool          m_delete;     ///< ノード差し替え時にメモリ削除するか
  self_type*    m_parent;     ///< 親ノード
  self_type*    m_next;       ///< 兄弟ノードの次
  self_type*    m_child_top;  ///< 子供ノード始端
  self_type*    m_child_end;  ///< 子供ノード終端
  value_type    m_value;      ///< 数値
};

} // namespace common

HIL_END_NAMESPACE;

#endif


