//==============================================================================
/**
 * @file    xml.hpp
 * @brief   XML操作
 * @author  H.I.
 * @date    2011-02-14
 * @version $Id: xml.hpp 45 2011-06-25 16:22:50Z com04 $
 */
//==============================================================================


#ifndef HIL_XML_HPP
#define HIL_XML_HPP


//==============================================================================
// include
//==============================================================================
#include <iostream>
#include <string>
#include "common.hpp"
#include "multi_tree.hpp"
#include "xml_node.hpp"

HIL_BEGIN_NAMESPACE;

namespace utility {

//==============================================================================
// define
//==============================================================================

//==============================================================================
/**
 * XML操作クラス
 */
//==============================================================================
class clXML {
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXML();
  
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   *
   * @param[ in ] hfilepath : 読み込みファイル
   */
  //-----------------------------------------------------------------------
  clXML(const std::string& hfilepath);
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  ~clXML();
  
  
  //-----------------------------------------------------------------------
  /**
   * ファイル読み込む
   *
   * @param[ in ] hfilepath : 読み込みファイル
   */
  //-----------------------------------------------------------------------
  void load(const std::string& hfilepath);
  
  //-----------------------------------------------------------------------
  /**
   * ファイル読み込めたか
   */
  //-----------------------------------------------------------------------
  bool isLoad() const;
  
  //-----------------------------------------------------------------------
  /**
   * ノードの検索
   *
   * @param[ in ] hstr  : 検索する文字列
   * return 見つかったノード
   */
  //-----------------------------------------------------------------------
  clXMLNode* find(const std::string& hstr);
  const clXMLNode* find(const std::string& hstr) const;
  
  //-----------------------------------------------------------------------
  /**
   * ノードの検索＋その直下の項目の取得
   *
   * @param[ in ] hstr  : 検索する文字列
   * return 見つかった項目
   */
  //-----------------------------------------------------------------------
  const std::string& getElement(const std::string& hstr) const;
  
  
  //-----------------------------------------------------------------------
  /**
   * トップノードの取得
   *
   * return トップノード
   */
  //-----------------------------------------------------------------------
  clXMLNode* getTopNode() { return &m_node; }
  const clXMLNode* getTopNode() const { return &m_node; }
  
  
  //-----------------------------------------------------------------------
  /**
   * データを表示する
   */
  //-----------------------------------------------------------------------
  void print() const;
  
  //-----------------------------------------------------------------------
  /**
   * ファイルに出力する
   *
   * @param[ in ] hfilepath : 出力ファイル
   */
  //-----------------------------------------------------------------------
  void exportFile(const std::string& hfilepath);
  
private:

  //-----------------------------------------------------------------------
  /**
   * データをパースする
   *
   * @param[ in ] hparent_node    : 親ノード
   * @param[ in ] hbuff           : データバッファ
   * @param[ in ] hindex          : hbuffを読み始めるインデックス
   */
  //-----------------------------------------------------------------------
  void parse(clXMLNode* hparent_node, const std::string& hbuff, size_t hindex);
  
  //-----------------------------------------------------------------------
  /**
   * タグデータをパースする
   *
   * @param[ out ] hend_index    : 読み終わったインデックス
   * @param[ in ]  hbuff         : データバッファ
   * @param[ in ]  hindex        : hbuffを読み始めるインデックス
   * @return                     : 生成したノード（外部でdeleteする事）
   */
  //-----------------------------------------------------------------------
  clXMLNode* parseTag(size_t* hend_index, const std::string& hbuff,
      size_t hindex);
  
  
  //-----------------------------------------------------------------------
  /**
   * タグデータをパースする
   *
   * @param[ out ] hattribute       : 設定する属性先
   * @param[ in ]  hattribute_size  : hattributeの要素数
   * @param[ in ]  hbegin           : タグの開始ポインタ
   * @param[ in ]  hend             : タグの終了ポインタ
   */
  //-----------------------------------------------------------------------
  void parseAttributes(clXMLNode::attribute_type* hattribute, size_t hattribute_size,
      const char* hbegin, const char* hend);
  
  //-----------------------------------------------------------------------
  /**
   * タグの属性データをパースする
   *
   * @param[ out ] hattribute_name    : 属性名
   * @param[ out ] hattribute_param   : 属性パラメータ
   * @param[ in ]  hbegin             : タグの開始ポインタ
   * @param[ in ]  hend               : タグの終了ポインタ
   */
  //-----------------------------------------------------------------------
  void parseAttribute(std::string* hattribute_name, std::string* hattribute_param,
      const char* hbegin, const char* hend);
  
  
private:
  clXMLNode   m_node;   ///< トップノード
};

} // namespace utility

HIL_END_NAMESPACE;

#endif


