//==============================================================================
/**
 * @file    xml_node.hpp
 * @brief   XMLノード
 * @author  H.I.
 * @date    2011-02-17
 * @version $Id: xml_node.hpp 29 2011-04-15 18:38:03Z blue $
 */
//==============================================================================


#ifndef HIL_XML_NODE_HPP
#define HIL_XML_NODE_HPP


//==============================================================================
// include
//==============================================================================
#include <iostream>
#include <string>
#include <vector>
#include "common.hpp"
#include "multi_tree.hpp"

HIL_BEGIN_NAMESPACE;

namespace utility {
//==============================================================================
// define
//==============================================================================

//==============================================================================
/**
 * XML用ノード
 */
//==============================================================================
class clXMLNode : public common::clMultiTree<std::string> {
public:
  /// ノード種類
  enum enType {
    TYPE_UNKNOWN,       ///< 不明
    TYPE_TAG,           ///< タグ
    TYPE_EMPTY_TAG,     ///< 空要素
    TYPE_ELEMENT,       ///< 項目
    TYPE_DECLARATION,   ///< 宣言
    TYPE_COMMENT,       ///< コメント
    
    TYPE_TERM
  };
  
public:
  typedef common::clMultiTree<std::string> super_type;
  typedef common::clMultiTree<std::string> tree_type;
  typedef std::pair<std::string, std::string> attribute_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXMLNode();
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clXMLNode();
  
  //-----------------------------------------------------------------------
  /**
   * 表示
   *
   * @param[ in ] hstream : 出力先ストリーム
   * @param[ in ] hindent : インデント
   */
  //-----------------------------------------------------------------------
  virtual void print(std::ostream* hstream=&std::cout, int hindent=0) const;
  
  //-----------------------------------------------------------------------
  /**
   * 属性を検索する
   *
   * @param[ in ] hattribute  : 検索する属性
   * @return !0               : 見つかった属性
   * @return 0                : 属性が見つからなかった
   */
  //-----------------------------------------------------------------------
  attribute_type* searchAttribute(const char* hattribute);
  const attribute_type* searchAttribute(const char* hattribute) const;
  
  
  //-----------------------------------------------------------------------
  /**
   * タグ設定
   *
   * @param[ in ] htag        : タグ名
   */
  //-----------------------------------------------------------------------
  void setTag(const std::string& htag);
  
  //-----------------------------------------------------------------------
  /**
   * 項目の設定
   *
   * @param[ in ] helement  : 項目
   */
  //-----------------------------------------------------------------------
  void setElement(const std::string& htag);
  
  //-----------------------------------------------------------------------
  /**
   * 属性設定
   *
   * @param[ in ] hattribute      : 属性リスト
   * @param[ in ] hattribute_size : hattributeの要素数
   */
  //-----------------------------------------------------------------------
  void setAttribute(const attribute_type* hattribute, size_t hattribute_size);
  
  //-----------------------------------------------------------------------
  /**
   * 属性設定
   *
   * @param[ in ] hattribute      : 属性リスト
   */
  //-----------------------------------------------------------------------
  void setAttribute(const std::vector<attribute_type>& hattribute);
  
  
  
  /// ノード種類の取得
  enType getType() const;
  /// タグの取得
  const std::string& getTag() const;
  /// 項目の取得
  const std::string& getElement() const;
  /// 属性リストの取得
  const std::vector<attribute_type>& getAttribute() const;
  
  
protected:
  /// ノード種類の設定
  void setType(enType htype);
  
private:
  enType                        m_type;       ///< ノード種類
  std::vector<attribute_type>   m_attribute;  ///< 属性リスト
};


//==============================================================================
/**
 * XML - タグ用ノード
 */
//==============================================================================
class clXMLNodeTag : public clXMLNode {
public:
  typedef clXMLNode super_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXMLNodeTag();
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clXMLNodeTag();
  
  //-----------------------------------------------------------------------
  /**
   * 表示
   *
   * @param[ in ] hstream : 出力先ストリーム
   * @param[ in ] hindent : インデント
   */
  //-----------------------------------------------------------------------
  virtual void print(std::ostream* hstream=&std::cout, int hindent=0) const;
  
  
private:
};


//==============================================================================
/**
 * XML - 空要素用ノード
 */
//==============================================================================
class clXMLNodeEmptyTag : public clXMLNode {
public:
  typedef clXMLNode super_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXMLNodeEmptyTag();
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clXMLNodeEmptyTag();
  
  //-----------------------------------------------------------------------
  /**
   * 表示
   *
   * @param[ in ] hstream : 出力先ストリーム
   * @param[ in ] hindent : インデント
   */
  //-----------------------------------------------------------------------
  virtual void print(std::ostream* hstream=&std::cout, int hindent=0) const;
  
  
private:
};


//==============================================================================
/**
 * XML - 項目用ノード
 */
//==============================================================================
class clXMLNodeElement : public clXMLNode {
public:
  typedef clXMLNode super_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXMLNodeElement();
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clXMLNodeElement();
  
  //-----------------------------------------------------------------------
  /**
   * 表示
   *
   * @param[ in ] hstream : 出力先ストリーム
   * @param[ in ] hindent : インデント
   */
  //-----------------------------------------------------------------------
  virtual void print(std::ostream* hstream=&std::cout, int hindent=0) const;
  
  
private:
};


//==============================================================================
/**
 * XML - 宣言用ノード
 */
//==============================================================================
class clXMLNodeDeclaration : public clXMLNode {
public:
  typedef clXMLNode super_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXMLNodeDeclaration();
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clXMLNodeDeclaration();
  
  //-----------------------------------------------------------------------
  /**
   * 表示
   *
   * @param[ in ] hstream : 出力先ストリーム
   * @param[ in ] hindent : インデント
   */
  //-----------------------------------------------------------------------
  virtual void print(std::ostream* hstream=&std::cout, int hindent=0) const;
  
  
private:
};


//==============================================================================
/**
 * XML - コメント用ノード
 */
//==============================================================================
class clXMLNodeComment : public clXMLNode {
public:
  typedef clXMLNode super_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXMLNodeComment();
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clXMLNodeComment();
  
  //-----------------------------------------------------------------------
  /**
   * 表示
   *
   * @param[ in ] hstream : 出力先ストリーム
   * @param[ in ] hindent : インデント
   */
  //-----------------------------------------------------------------------
  virtual void print(std::ostream* hstream=&std::cout, int hindent=0) const;
  
  
private:
};

} // namespace utility

HIL_END_NAMESPACE;

#endif


